package com.recordingblogs.exampledelay;

import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import javax.sound.sampled.AudioFormat;
import javax.swing.*;
import javax.swing.event.*;
import com.recordingblogs.oreffect.*;

public class DelayPanel extends JPanel implements EffectPanelInterface, ActionListener, FocusListener, ChangeListener
{
	/**
	 * This serial version UID is not used by Orinj
	 */
	private static final long serialVersionUID = -2448575955941157613L;

	/**
	 * A text field to contain the amount of delay in milliseconds
	 */
	private JTextField m_delayField;
	
	/**
	 * A text field to contain the amount of decay in percentages (i.e., 80%
	 * means that the amplitude of the delayed signal will be 80% of the 
	 * amplitude of the original signal)
	 */
    private JTextField m_decayField;
    
    /**
     * A slider for the amount of delay in milliseconds
     */
    private JSlider m_delaySlider;
    
    /** 
     * A slider for the amount of decay in percentages
     */
    private JSlider m_decaySlider;

    /**
     * A reference to the actual object of class Delay.  This reference is
     * set in the DelayPanel constructor below
     */
    private Delay m_delay;

    /**
     * The labels for the sliders.  This is not necessary, but is an easy 
     * way to set the font and foreground of the sliders.  Setting the fonts
     * and foreground is not necessary
     */
    public class SliderLabel extends JLabel
    {
    	/**
    	 * This serial version UID is not used
    	 */
		private static final long serialVersionUID = 5206127030980635365L;

		/**
		 * The one and only constructor
		 * @param string The text of the label
		 */
		public SliderLabel(String string)
        {
            super(string);
            setFont(EffectFont.SMALLFONT);
        }
    }

    /**
     * The focus policy for the panel.  A focus policy is not required, 
     * but is usually a good idea.  Note that this is just the focus policy for
     * this panel, but this panel will be contained in a larger dialog.  The larger
     * dialog, which is a part of Orinj, will define its focus policy taking
     * into account the order of the controls specified here 
     */
    private class FocusPolicy extends ContainerOrderFocusTraversalPolicy
    {
    	/**
    	 * This serial version UID is not used
    	 */
		private static final long serialVersionUID = 1586957514701977623L;

		@Override
		public Component getDefaultComponent(Container c)
        {
            return m_delayField;
        }

		@Override
        public Component getLastComponent(Container c)
        {
            return m_decaySlider;
        }

		@Override
        public Component getFirstComponent(Container c)
        {
            return m_delayField;
        }

		@Override
        public Component getComponentBefore(Container c, Component a)
        {
            if(a == m_decaySlider)
                return m_decayField;
            if(a == m_decayField)
                return m_delaySlider;
            if(a == m_delaySlider)
                return m_delayField;
            if (a == m_delayField)
            	return m_decaySlider;
            return m_delayField;
        }

		@Override
        public Component getComponentAfter(Container c, Component a)
        {
            if(a == m_delayField)
                return m_delaySlider;
            if(a == m_delaySlider)
                return m_decayField;
            if(a == m_decayField)
                return m_decaySlider;
            if(a == m_decaySlider)
                return m_delayField;
            return m_delayField;
        }
    }

    /**
     * The one and only constructor.  The two arguments in this constructor
     * are required for each effect.
     * @param delay The delay effect.  This argument will obviously be of a 
     *              different class for each effect
     * @param format This audio format is not used in this effect.  Audio formats
     *               are typically used in the graphical user interfaces in 
     *               equalizer type effects and primarily for the purpose of 
     *               drawing the actual magnitude response of the equalizers at
     *               various frequencies.  Note that this audio format is not the
     *               audio format that is sent to the apply function of the actual
     *               effect
     */
    public DelayPanel(Delay delay, AudioFormat format)
    {
        GridBagLayout layout = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        setLayout(layout);

        // note that we have a choice on whether to set the foreground of 
        // this label or not.  While we are setting the foreground of slider labels
        // we are not doing so here.  This means that, in darker skins of Orinj,
        // the slider labels will be white, while this label will remain black.
        // This is a choice for the designer
        JLabel delayLabel = new JLabel("Delay (ms):");	//$NON-NLS-1$
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.weightx = 1;
        constraints.weighty = 0;
        constraints.fill = GridBagConstraints.BOTH;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(delayLabel, constraints);
        add(delayLabel);

        m_delayField = new JTextField(4);
        m_delayField.setToolTipText("Set the amount of delay");	//$NON-NLS-1$
        m_delayField.addActionListener(this);
        m_delayField.addFocusListener(this);
        constraints.gridy = 1;
        layout.setConstraints(m_delayField, constraints);
        add(m_delayField);

        // delay values are in milliseconds
        Hashtable<Integer, SliderLabel> numbersDelay = new Hashtable<>();
        numbersDelay.put(Integer.valueOf(0), new SliderLabel("0"));	//$NON-NLS-1$
        numbersDelay.put(Integer.valueOf(200), new SliderLabel("200"));	//$NON-NLS-1$
        numbersDelay.put(Integer.valueOf(400), new SliderLabel("400"));	//$NON-NLS-1$
        numbersDelay.put(Integer.valueOf(600), new SliderLabel("600"));	//$NON-NLS-1$
        numbersDelay.put(Integer.valueOf(800), new SliderLabel("800"));	//$NON-NLS-1$
        numbersDelay.put(Integer.valueOf(1000), new SliderLabel("1000"));	//$NON-NLS-1$

        m_delaySlider = new JSlider(SwingConstants.VERTICAL, (int) (Delay.MINDELAY * 1000), (int) (Delay.MAXDELAY * 1000), 0);
        m_delaySlider.setMajorTickSpacing(200);
        m_delaySlider.setMinorTickSpacing(50);
        m_delaySlider.setPaintTicks(true);
        m_delaySlider.setPaintLabels(true);
        m_delaySlider.setLabelTable(numbersDelay);
        m_delaySlider.setToolTipText("Set the amount of delay");	//$NON-NLS-1$
        m_delaySlider.addChangeListener(this);
        m_delaySlider.addFocusListener(this);
        constraints.gridy = 3;
        constraints.weighty = 1.0D;
        layout.setConstraints(m_delaySlider, constraints);
        add(m_delaySlider);

        JLabel decayLabel = new JLabel("Decay (%):");	//$NON-NLS-1$
        constraints.gridx = 1;
        constraints.gridy = 0;
        constraints.insets.left = 5;
        constraints.weighty = 0;
        layout.setConstraints(decayLabel, constraints);
        add(decayLabel);

        m_decayField = new JTextField(4);
        m_decayField.setToolTipText("Set the amount of decay");	//$NON-NLS-1$
        m_decayField.addActionListener(this);
        m_decayField.addFocusListener(this);
        constraints.gridy = 1;
        layout.setConstraints(m_decayField, constraints);
        add(m_decayField);

        Hashtable<Integer, SliderLabel> numbersDecay = new Hashtable<>();
        numbersDecay.put(Integer.valueOf(0), new SliderLabel("0"));		//$NON-NLS-1$
        numbersDecay.put(Integer.valueOf(20), new SliderLabel("20"));		//$NON-NLS-1$
        numbersDecay.put(Integer.valueOf(40), new SliderLabel("40"));		//$NON-NLS-1$
        numbersDecay.put(Integer.valueOf(60), new SliderLabel("60"));		//$NON-NLS-1$
        numbersDecay.put(Integer.valueOf(80), new SliderLabel("80"));		//$NON-NLS-1$
        numbersDecay.put(Integer.valueOf(100), new SliderLabel("100"));		//$NON-NLS-1$

        m_decaySlider = new JSlider(SwingConstants.VERTICAL, (int) (Delay.MINDECAY * 100), (int) (Delay.MAXDECAY * 100), 100);
        m_decaySlider.setMajorTickSpacing(20);
        m_decaySlider.setMinorTickSpacing(5);
        m_decaySlider.setPaintTicks(true);
        m_decaySlider.setPaintLabels(true);
        m_decaySlider.setLabelTable(numbersDecay);
        m_decaySlider.setToolTipText("Set the amount of decay");	//$NON-NLS-1$
        m_decaySlider.addChangeListener(this);
        m_decaySlider.addFocusListener(this);
        constraints.gridy = 3;
        constraints.weighty = 1;
        layout.setConstraints(m_decaySlider, constraints);
        add(m_decaySlider);

        // set the reference to the effect that this graphical user
        // interface should control
        m_delay = delay;
        
        setFocusTraversalPolicy(new FocusPolicy());
        
        // set the controls of the graphical user interface according to the 
        // data of the effect
        updateData();
    }

    /**
     * Capture change events.  In this effect, change events will be fired by
     * the two sliders.  Note that this function does not store undo information.
     * We could store undo on every movement of the slider, but that will probably
     * be too much.  Rather, we will store undo information only when the user
     * moves away from the slider (see focusLost below).  The updateData calls 
     * here ensure that the delay and decay text field values change when the slider
     * moves
     * @param e The change event
     */
    @Override
    public void stateChanged(ChangeEvent e)
    {
        if(e.getSource() == m_delaySlider)
        {
            m_delay.setDelay(m_delaySlider.getValue() / 1000F, false);
            updateData();
        }
        else if(e.getSource() == m_decaySlider)
        {
            m_delay.setDecay(m_decaySlider.getValue() / 100F, false);
            updateData();
        }
    }

    /**
     * Capture action events.  Action events will be fired by the two text
     * fields.  Note that we check for text field errors (i.e., the user entered
     * the wrong number or a non-number), but we do not show error messages.  We 
     * simply move the focus away from the text field and let the focusLost
     * function handle the error messages.  We have a choice in how we implement
     * error checks and this is simply one option.
     * Note also that we choose to not store undo information on action events. 
     * Thus, the setDelay and setDecay functions are called with a second argument
     * equal to false.  We choose to only store undo information on focus lost.
     * This is, again, a choice and could be done differently.
     * @param e The action event
     */
    @Override
    public void actionPerformed(ActionEvent e)
    {
        if(e.getSource() == m_delayField)
        {
            try
            {
            	float value = Float.parseFloat(m_delayField.getText()) / 1000F;
            	if (value < Delay.MINDELAY || value > Delay.MAXDELAY)
            	{
                    m_delaySlider.grabFocus();
                    return;
            	}
	            m_delay.setDelay(value, false);
	            updateData();
            }
            catch (NumberFormatException ex)
            {
                m_delaySlider.grabFocus();
            }
        }
        else if(e.getSource() == m_decayField)
        {
        	try
        	{
            	float value = Float.parseFloat(m_decayField.getText()) / 100F;
            	if (value < Delay.MINDECAY || value > Delay.MAXDECAY)
            	{
                    m_decaySlider.grabFocus();
                    return;
            	}
	            m_delay.setDecay(value, false);
	            updateData();
        	}
        	catch (NumberFormatException ex)
        	{
                m_decaySlider.grabFocus();
        	}
        }
    }

    /**
     * Capture focus gained events.  There is nothing to do in this function
     * @param e The focus event
     */
    @Override
    public void focusGained(FocusEvent e)
    {
    	// nothing to do
    }

    /**
     * Capture focus lost events. These events will be fired by all four components
     * of the graphical user interface for this effect (but, for example, probably
     * will not be useful for checkbox type controls, if the interface had any).
     * Note first that this function throws error messages when the user enters
     * a number outside of the allowed interval or enters a non-number.  Note also
     * that this function does store undo information, when the effect controls change.
     * Thus, the setDelay and setDecay functions are called with a second argument
     * equal to true.
     * @param e The focus event
     */
    @Override
    public void focusLost(FocusEvent e)
    {
        if(e.getSource() == m_delayField)
        {
        	try
        	{
            	float value = Float.parseFloat(m_delayField.getText()) / 1000F;
            	if (value < Delay.MINDELAY || value > Delay.MAXDELAY)
            	{
                    JOptionPane.showMessageDialog(this, "Please enter a number between "	//$NON-NLS-1$
                    		+ (int) (Delay.MINDELAY * 1000) + " and "	//$NON-NLS-1$
                    		+ (int) (Delay.MAXDELAY * 1000), "Error", JOptionPane.ERROR_MESSAGE);	//$NON-NLS-1$
                    m_delayField.grabFocus();
                    updateData();
                    return;
            	}
	            m_delay.setDelay(value, true);
	            updateData();
        	}
        	catch (NumberFormatException ex)
        	{
                JOptionPane.showMessageDialog(this, "Please enter a valid number", 	//$NON-NLS-1$
                		"Error", JOptionPane.ERROR_MESSAGE);	//$NON-NLS-1$
	            updateData();
        		m_delayField.grabFocus();
        	}
        }
        else if(e.getSource() == m_decayField)
        {
        	try
        	{
            	float value = Float.parseFloat(m_decayField.getText()) / 100F;
            	if (value < Delay.MINDECAY || value > Delay.MAXDECAY)
            	{
                    JOptionPane.showMessageDialog(this, "Please enter a number between "	//$NON-NLS-1$
                    		+ (int) (Delay.MINDECAY * 100) + " and "	//$NON-NLS-1$
                    		+ (int) (Delay.MAXDECAY * 100), "Error", JOptionPane.ERROR_MESSAGE);	//$NON-NLS-1$
                    m_decayField.grabFocus();
                    updateData();
                    return;
            	}
	            m_delay.setDecay(value, true);
	            updateData();
        	}
        	catch (NumberFormatException ex)
        	{
                JOptionPane.showMessageDialog(this, "Please enter a valid number", 	//$NON-NLS-1$
                		"Error", JOptionPane.ERROR_MESSAGE);	//$NON-NLS-1$
	            updateData();
        		m_decayField.grabFocus();
        	}
        }
        else if(e.getSource() == m_delaySlider)
        {
            m_delay.setDelay(m_delaySlider.getValue() / 1000F, true);
        }
        else if(e.getSource() == m_decaySlider)
        {
            m_delay.setDecay(m_decaySlider.getValue() / 100F, true);
        }
    }

    /**
     * Set the graphical user interface controls equal to the data of the effect
     */
    @Override
    public void updateData()
    {
        m_delaySlider.setValue((int)(m_delay.getDelay() * 1000F));
        m_delayField.setText(Integer.toString((int)(m_delay.getDelay() * 1000F)));
        m_decaySlider.setValue((int)(m_delay.getDecay() * 100F));
        m_decayField.setText(Integer.toString((int)(m_delay.getDecay() * 100F)));
    }
}
